#! /bin/env python

import numpy as np
import matplotlib.pylab as plt
import pacsplot as pp
import carmatable as ct
import os

def baselineplot(col,array,antpos,project,cFreq,sFreq):
    """
        Creates a baseline phase versus time plot with the data from the column that the user inputs. Also creates a txt
    file that has information about the start times, end times, sources, start run times, and end run times of the time
    intervals.
    Input:
    col: The column of the carmaphaseDiffs.txt and szaphaseDiffs.txt files that the program will use the data for
    and make the plot for. Each column of these files has a baseline pair.

    Outputs:
    PLOT: Baseline phase versus time plot
    timesOutput: the name of the file that the times and sources information will output to
    """
    scalar = float(cFreq)/float(sFreq)
    
    tableInc = ct.CarmaTable()
    tableInc.open("carmaphaseDiffs.txt")
    tableIns = ct.CarmaTable()
    tableIns.open("szaphaseDiffsInterpolated.txt")
    # both 's' and 'c' are the same now
    times = tableIns.getCol(0)
    #stripping off the unmatched carma times
    for i in range(-tableInc.getNrows(),0):
        if tableInc.getRow(i)[0] not in times:
            tableInc.removeRow(i)
    cColdata = tableInc.getCol(col)
    sColdata = tableIns.getCol(col)
    cHeaderwords = tableInc.getColNames()
    sHeaderwords = tableIns.getColNames()
    
    seconds = []
    index = []
    index2 = []
    count2 = 0
    for time in times:
        seconds.append(pp.timeToSec(time))
        index.append(count2)
        count2 = count2 + 4
    #converting H:M:S times into seconds, and counting the number of times
    # PJT: what's this '4' in the count2 increment?

    sColdataScaled = []
    for element in sColdata:
        degree = float(element) * scalar
        #degree = wdg.wrapDegree(degree)
        sColdataScaled.append(degree)
    #scaling sza phase values by scalar (cFreq/sFreq)

    ###BASELINE LENGTH CALC
    #cblLen = pp.baselineLength(cHeaderwords[col], "carma")
    sblLen = pp.baselineLength(sHeaderwords[col], "sza")
    
    cblLen = pp.baselineLength(cHeaderwords[col], "carma",antpos)
    #sblLen = pp.baselineLength(sHeaderwords[col], "sza",antpos)
    ##########################

    title = "CARMA " + cHeaderwords[col] + "(" + str(int(cblLen)) + " m) and SZA " + sHeaderwords[col] + "(" + str(int(sblLen)) + " m)"
    #plt.plot(seconds, cColdata, 'g--')
    plt.plot(index, cColdata, 'g-')
    #plt.plot(seconds, sColdataScaled, 'b-')

    #print len(index)
    #print len(sColdataScaled)
    plt.plot(index, sColdataScaled, 'b--')

    ######### Adding Seperator Lines ### AND ###
    ############Outputting to text file what interval corresponds to what index ########## 
    threshold = 15 #number of seconds that will cause program to think that time interval is a new one
    starttimes = []
    endtimes = []
    oldTime = "00:00:00.0"

    for time in times:
        if oldTime == "00:00:00.0":
            starttimes.append(time)
        elif (pp.timeToSec(time) - pp.timeToSec(oldTime)) > threshold:
            endtimes.append(oldTime)
            starttimes.append(time)
        oldTime = time
    endtimes.append(time)
    #calculating start and end times with carmaphaseDiffs.txt times, and threshold
    starttimes2 = []
    endtimes2 = []
    for (sTime,eTime) in zip(starttimes,endtimes):
        starttimes2.append(pp.timeToSec(sTime))
        endtimes2.append(pp.timeToSec(eTime))
    #convert the times to seconds in the new lists

    diffTimes = []
    for (sTime,eTime) in zip(starttimes2,endtimes2):
        diffTimes.append(eTime - sTime)

    runtimes = []
    totalD = 0
    for diff in diffTimes:
        totalD = totalD + diff
        runtimes.append(totalD)
        #totalD keeps track of the total difference in time to allow the plot to cumulatively plot the seperators
        #from the differences between the start times and end times
        plt.plot([totalD,totalD], [plt.axis()[2],plt.axis()[3]], 'k--') #makes a line as tall as current axis

    ############################################################################################

    #ATTEMPTS AT A DIFFERENCE PLOT. (AFTER-CORRECTION PLOT)
    #diffCS = []
    #for (c,s) in zip(cColdata, sColdataScaled):
    #    curDiff = int(c) - int(s)
    #    diffCS.append(curDiff)
    #plt.plot(seconds, diffCS, 'r-.')
    
    plt.title(title, fontsize=12)

    #############################
    
    #plt.xlabel('Time (seconds)')
    plt.xlabel('index * 4 (Run Time Seconds)')
    plt.ylabel('Phase (degrees)')
    plt.legend(('CARMA','SZA * %.3f'%(scalar)),'upper right')

    return (starttimes,endtimes,runtimes,threshold)
    
def plots(array,antpos,project,cFreq,sFreq,output="finalTimes/finalTimeHeader.txt"):
    
    count = 1
    while (count <= 28):
        plt.figure(num=1, figsize=(14,11), dpi=90)
        (starttimes,endtimes,runtimes,threshold) = baselineplot(count,array,antpos,project,cFreq,sFreq)
        plt.savefig("bltime%03i.pdf"%(count), orientation='landscape', bbox_inches='tight', pad_inches=1)
        plt.close(1)
        count = count + 1

    (topsstarts,topsends,sources) = pp.getTimes("%s.txt"%(project))

    ####
    topsstarts = pp.midnight(topsstarts)
    topsends = pp.midnight(topsends)
    ###Correcting for times over midnight

    count = 0
    oldend = ""
    newstarts = []
    newends = []
    for (topsstart,topsend) in zip(topsstarts,topsends):
        if count == 0:
            newstarts.append(topsstart)
            newends.append(topsend)
        elif (pp.timeToSec(topsstart) - pp.timeToSec(oldend)) > threshold:
            newstarts.append(topsstart)
            newends.append(topsend)
        else:
            print "Combining time interval from time ops output (within memory) due to lower than threshold of" + str(threshold)
            del newends[-1]
            newends.append(topsend)
        #oldstart = topsstart
        oldend = topsend
        count = count + 1
    topsstarts = newstarts[:]
    topsends = newends[:]
    #combining tops intervals that have less than threshold seconds between them


    newSources = []
    for (starttime,endtime) in zip(starttimes,endtimes):
        for (topsstart,topsend,source) in zip(topsstarts,topsends,sources):
            #print (topsstart,starttime,endtime,topsend)
            if (pp.timeToSec(topsstart) <= pp.timeToSec(starttime) <= pp.timeToSec(endtime) <= pp.timeToSec(topsend)):
                newSources.append(source)
    #calculating sources within SEPARATOR time intervals

    sources = newSources[:]
    
    starttimes2 = []
    endtimes2 = []
    for (sTime,eTime) in zip(starttimes,endtimes):
        starttimes2.append(pp.timeToSec(sTime))
        endtimes2.append(pp.timeToSec(eTime))
    #convert the times to seconds in the new lists

    diffTimes = []
    for (sTime,eTime) in zip(starttimes2,endtimes2):
        diffTimes.append(eTime - sTime)

    runtimes = []
    totalD = 0
    for diff in diffTimes:
        totalD = totalD + diff
        runtimes.append(totalD)
        #totalD keeps track of the total difference in time to allow the plot to cumulatively plot the seperators
        #from the differences between the start times and end times
        plt.plot([totalD,totalD], [plt.axis()[2],plt.axis()[3]], 'k--') #makes a line as tall as current axis

    startruns = []
    endruns = []
    startruns.append(0)
    for time in runtimes:
        endruns.append(time)
        startruns.append(time)
    del startruns[len(startruns) - 1]

    names = ["Interval", "Start Time(H:M:S)", "End Time(H:M:S)", "Source", "Start Run Time (Seconds)", "End Run Time (Seconds)"]
    types = ["i","hms","hms","s","r","r"]
    units = ["","","","","seconds","seconds"]
    tableOut = ct.CarmaTable(names,types,units)
    indexes = []
    count = 1
    for element in starttimes:
        indexes.append(count)
        count = count + 1

    #############################
    
    tableOut.editCol(indexes,0)
    tableOut.editCol(starttimes,1)
    tableOut.editCol(endtimes,2)
    tableOut.editCol(sources,3)
    tableOut.editCol(startruns,4)
    tableOut.editCol(endruns,5)
    """
    #(instarts,inends,intimes,inrmslist) = pp.calcRMSPath(starttimes,endtimes)
    (instarts,inends,intimes,inrmslist) = pp.calcWeatherVar(starttimes,endtimes,"rmspath")    
    count = 0
    temprmslist = []
    avgrmslist = []
    prevstart = ""
    for (instart) in (instarts):
        if count > 0:
            if instart == prevstart:
                temprmslist.append(inrmslist[count])
            else:
                avgrmslist.append(pp.avg(temprmslist))
                temprmslist = []
                temprmslist.append(inrmslist[count])
        else:
            temprmslist.append(inrmslist[count])
        prevstart = instart        
        count = count + 1
    avgrmslist.append(pp.avg(temprmslist))
    tableOut.addCol("Average RMS Path","d","microns",6)
    print avgrmslist
    tableOut.editCol(avgrmslist,6)
    ####Adding RMSPATH!!!!
    """
    os.system("mkdir -p finalTimes")
    tableOut.write(output)
    #writing out the times and sources info to the file

